@@ -29,12 +29,14 @@ from .processor import (
     EmitCommand,
     Processor,
 )
-from .common import JembeInitParamSupport, exec_name_to_full_name, get_annotation_type
+from .common import (
+    JembeInitParamSupport,
+    exec_name_to_full_name,
+    get_annotation_type,
+    DisplayResponse,
+)
 from .files import Storage
 
-if TYPE_CHECKING:  # pragma: no cover
-    from flask import Response
-
 
 class ComponentState(dict):
     """
@@ -107,8 +109,48 @@ class ComponentReference:
     {{component('name').is_accessible}}..{{component().jrl}}...
     Add comand 'init' with same usage as component but without
     using state params of exisiting component if its on page ??
+
+
+    factory class method allow use of {{component('../name')}} or {{component('/a/b/c')}}
     """
 
+    @classmethod
+    def factory(
+        cls,
+        caller_exec_name: Optional[str],
+        name: str,
+        kwargs: dict,
+        merge_existing_params: bool = True,
+    ) -> "ComponentReference":
+        name_split = name.split("/")
+        cr: Optional["ComponentReference"] = None
+        if (name_split[0] == "" and len(name_split) > 2) or (  # /a/b
+            name_split[0] != "" and len(name_split) > 1
+        ):  # a/b ../b
+            if name_split[0] == "":
+                name_split = name_split[1:]
+                name_split[0] = "/{}".format(name_split[0])
+            for pname in name_split[:-1]:
+                if cr is None:
+                    cr = cls(
+                        caller_exec_name,
+                        pname,
+                        {},
+                        merge_existing_params,
+                    )
+                else:
+                    cr = cr.component(pname)
+            cr = (
+                cr.component(name_split[-1], **kwargs)
+                if cr is not None
+                else cls(
+                    caller_exec_name, name_split[-1], kwargs, merge_existing_params
+                )
+            )
+        else:
+            cr = cls(caller_exec_name, name, kwargs, merge_existing_params)
+        return cr
+
     def __init__(
         self,
         caller_exec_name: Optional[str],
@@ -119,11 +161,18 @@ class ComponentReference:
         """
         - name can be name of subcomponent or if it starts with '/' name of root component
         """
+        self._key = ""
+
         if name == "":
             raise JembeError("Component name can't be empty string")
+
+        if name == "." and caller_exec_name is None:
+            raise JembeError(
+                "ComponentReference can't be called  to reference self without caller_exec_name specified"
+            )
+
         self.caller_exec_name = caller_exec_name
         self.name = name
-        self._key = ""
         self.action = ComponentConfig.DEFAULT_DISPLAY_ACTION
         self.action_args: Tuple[Any, ...] = ()
         self.action_kwargs: dict = {}
@@ -134,21 +183,23 @@ class ComponentReference:
         self._component_instance: Optional["Component"] = None
         self._is_accessible: Optional[bool] = None
 
-        if "." in self.name or (
-            "/" in self.name
-            and not self.name.startswith("/")
-            and len(self.name.split("/")) == 2
-        ):
-            raise JembeError(
-                "Component renderer only supports rendering and accessing direct childs or root page component"
-            )
         if self.caller_exec_name is None and not self.name.startswith("/"):
             raise JembeError(
                 "ComponentReference can't be called with relative path without caller_exec_name specified"
             )
 
+        if (".." in self.name and self.name != "..") or (
+            "/" in self.name
+            and not self.name.startswith("/")
+            # and len(self.name.split("/")) == 2
+        ):
+            raise JembeError(
+                "Component reference only supports rendering and accessing direct childs, root page or parent component"
+            )
+
         self.root_renderer = self
         self.active_renderer = self
+        self._aditional_components: List["Component"] = []
         self.base_jrl = "$jmb"
 
     def _component(
@@ -159,49 +210,89 @@ class ComponentReference:
         )
         cr.root_renderer = self.root_renderer
         cr.root_renderer.active_renderer = cr
+        cr._aditional_components = self._aditional_components.copy()
+        if (
+            self.component_instance is not None
+            and self.component_instance.exec_name not in self.processor.components
+        ):
+            cr._aditional_components.append(self.component_instance)
         cr.base_jrl = self.jrl
         return cr
 
     def component(self, jmb_exec_name: str, **kwargs) -> "ComponentReference":
-        return self._component(self.merge_existing_params, jmb_exec_name, **kwargs)
+        if self.action != ComponentConfig.DEFAULT_DISPLAY_ACTION:
+            raise JembeError(
+                "<{}> Cant reference child component when action '{}' is called".format(
+                    str(self), self.action
+                )
+            )
+        return self._component(True, jmb_exec_name, **kwargs)
 
     def component_reset(self, jmb_exec_name: str, **kwargs) -> "ComponentReference":
         return self._component(False, jmb_exec_name, **kwargs)
 
-    def component_merge(self, jmb_exec_name: str, **kwargs) -> "ComponentReference":
-        return self._component(True, jmb_exec_name, **kwargs)
-
     def _init_component(self):
         if self._component_initialise_done:
             return
         self._component_initialise_done = True
 
-        initialise_command = InitialiseCommand(
-            self.exec_name, self.kwargs, self.merge_existing_params
-        )
-        (
-            self._is_accessible,
-            self._component_instance,
-        ) = self.processor.execute_initialise_command_successfully(initialise_command)
+        if not (self.name == "." and self.root_renderer == self):
+            initialise_command = InitialiseCommand(
+                self.exec_name, self.kwargs, self.merge_existing_params
+            )
+            (
+                self._is_accessible,
+                self._component_instance,
+            ) = self.processor.execute_initialise_command_successfully(
+                initialise_command, *self._aditional_components
+            )
 
     @property
     def component_instance(self) -> Optional["Component"]:
-        self._init_component()
-        return self._component_instance
+        if self.name == "." and self.root_renderer == self:
+            return self.processor.components[self.exec_name]
+        else:
+            self._init_component()
+            return self._component_instance
 
     @property
     def is_accessible(self) -> bool:
         # TODO add param ignore_incoplete_params = True so that exception trown during initialise
         # becouse not all required init parameters are suplied are treated as not accessible (
         # catch exception and return False in this case)
-        self._init_component()
-        return self._is_accessible == True
+        if self.action != ComponentConfig.DEFAULT_DISPLAY_ACTION:
+            ci = self.component_instance
+            if ci is None:
+                current_app.logger.warning(
+                    "Component '{}' can't acuire referenced component '{}' to check if action '{}' is accessible".format(
+                        self.root_renderer.caller_exec_name, self.exec_name, self.action
+                    )
+                )
+                return True
+            return self.action not in ci._jembe_disabled_actions
+        else:
+            if self.name == ".":
+                return True
+            self._init_component()
+            return (
+                self._is_accessible == True
+                and self._component_instance is not None
+                and self._component_instance.ac_check()
+            )
 
     @cached_property
     def url(self) -> str:
         if not self.is_accessible:
             raise NotFound()
-        return self.component_instance.url  # type: ignore
+        if self._aditional_components:
+            backup_components = self.processor.components.copy()
+            for acomp in self._aditional_components:
+                self.processor.components[acomp.exec_name] = acomp
+            url = self.component_instance.url  # type: ignore
+            self.processor.components = backup_components
+            return url
+        else:
+            return self.component_instance.url  # type: ignore
 
     @cached_property
     def jrl(self) -> str:
@@ -215,13 +306,31 @@ class ComponentReference:
                 return v
             return "'{}'".format(v)
 
-        jrl = "component{reset}('{name}'{kwargs}{key}){action}".format(
-            reset="_reset" if not self.merge_existing_params else "",
-            name=self.name,
-            key=",key='{}'".format(self._key) if self._key else "",
-            action=".call('{name}',{{{kwargs}}},[{args}])".format(
+        jrl = (
+            "component{reset}('{name}'{kwargs}{key})".format(
+                reset="_reset" if not self.merge_existing_params else "",
+                name=self.name,
+                key=",key='{}'".format(self._key) if self._key else "",
+                kwargs=",{{{}}}".format(
+                    ",".join(
+                        (
+                            "{}:{}".format(k, _prep_v(v))
+                            for k, v in self.kwargs.items()
+                            if not k.startswith("_")
+                        )
+                    )
+                )
+                if self.kwargs
+                else "",
+            )
+            if self.name != "."
+            else ""
+        )
+        jrl += (
+            # ".call('{name}',{{{kwargs}}},[{args}])".format(
+            ".call('{name}',{{{kwargs}}})".format(
                 name=self.action,
-                args=",".join((_prep_v(v) for v in self.action_args)),
+                # args=",".join((_prep_v(v) for v in self.action_args)),
                 kwargs=",".join(
                     (
                         "{}:{}".format(k, _prep_v(v))
@@ -230,19 +339,10 @@ class ComponentReference:
                 ),
             )
             if self.action != ComponentConfig.DEFAULT_DISPLAY_ACTION
-            else ".display()",
-            kwargs=",{{{}}}".format(
-                ",".join(
-                    (
-                        "{}:{}".format(k, _prep_v(v))
-                        for k, v in self.kwargs.items()
-                        if not k.startswith("_")
-                    )
-                )
-            )
-            if self.kwargs
-            else "",
+            else ".display()"
         )
+        if jrl.startswith("."):
+            jrl = jrl[1:]
         base_jrl = (
             self.base_jrl[:-10]
             if self.base_jrl.endswith(".display()")
@@ -252,8 +352,17 @@ class ComponentReference:
 
     @cached_property
     def exec_name(self) -> str:
-        if self.name.startswith("/"):
+        if self.name == ".":
+            return self.caller_exec_name  # type:ignore
+        elif self.name.startswith("/"):
             return Component._build_exec_name(self.name.split("/")[1], self._key)
+        elif self.caller_exec_name is not None and self.name == "..":
+            caller_exec_name_split = self.caller_exec_name.split("/")
+            return Component._build_exec_name(
+                caller_exec_name_split[-2],
+                self._key,
+                "/".join(caller_exec_name_split[:-2]),
+            )
         elif self.caller_exec_name is not None:
             return Component._build_exec_name(
                 self.name, self._key, self.caller_exec_name
@@ -321,7 +430,7 @@ def component(
     jmb_exec_name: str, jmb_reset: bool = True, **kwargs
 ) -> "ComponentReference":
     """Creates component renderer that can be used to obtain any component url, jrl or check if it is accessible"""
-    return ComponentReference(None, jmb_exec_name, kwargs, not jmb_reset)
+    return ComponentReference.factory(None, jmb_exec_name, kwargs, not jmb_reset)
 
 
 def componentInitDecorator(init_method):
@@ -355,8 +464,15 @@ def componentInitDecorator(init_method):
                 }
             )
             self.state._injected_params_names = self._jembe_injected_params_names
-        # TODO raise Jembe error if not all required params are present
-        init_method(self, *args, **kwargs)
+        try:
+            init_method(self, *args, **kwargs)
+        except Exception as e:
+            current_app.logger.warning(
+                "{}: {};  args={}; kwargs={};".format(
+                    self.__class__.__name__, e, args, kwargs
+                )
+            )
+            raise e
 
     return decoratedInit
 
@@ -385,6 +501,18 @@ class ComponentMeta(ABCMeta):
                 and not p.name.startswith("_")
             }
             attrs["__init__"] = componentInitDecorator(attrs["__init__"])
+
+        # check if inject_into is overriden
+        inject_into_overriden = False
+        if "inject_into" in attrs:
+            inject_into_overriden = True
+        else:
+            for b in bases:
+                if b != Component and getattr(b, "_jembe_inject_into_overriden", False):
+                    inject_into_overriden = True
+                    break
+        attrs["_jembe_inject_into_overriden"] = inject_into_overriden
+
         new_class = super().__new__(cls, name, bases, dict(attrs), **kwargs)
         return new_class
 
@@ -394,6 +522,7 @@ class Component(metaclass=ComponentMeta):
     def _jembe_init_(
         cls,
         _config: ComponentConfig,
+        _component_exec_name: str,
         _jembe_injected_params_names: List[str],
         _jembe_merged_existing_params: bool,
         **init_params
@@ -402,14 +531,16 @@ class Component(metaclass=ComponentMeta):
         Instance creation by explicitly calling __new__ and __init__
         because _config should be avaiable in __init__
         """
-        component = object.__new__(cls)
-        component._config = _config
+        component: "Component" = object.__new__(cls)
+        component._config = _config  # type: ignore
         component._jembe_injected_params_names = _jembe_injected_params_names
         component._jembe_merged_existing_params = _jembe_merged_existing_params
-        component.__init__(**init_params)
+        component._jembe_disabled_actions = []
+        component.exec_name = _component_exec_name
+        component.__init__(**init_params)  # type: ignore
+        component.init()
         return component
 
-    state: "ComponentState"
     _jembe_init_signature: "Signature"
     _jembe_init_param_names: Tuple[str, ...]
     _jembe_state_param_names: Tuple[str, ...]
@@ -417,16 +548,32 @@ class Component(metaclass=ComponentMeta):
     _jembe_injected_params_names: List[str]
     _jembe_config_init_params: Dict[str, Any]
     _jembe_merged_existing_params: bool
+    _jembe_inject_into_overriden: bool
     _config: "Config"
 
+    state: "ComponentState"
+    _jembe_disabled_actions: List[str]
+
     class Config(ComponentConfig):
         pass
 
     def __init__(self):
-        self.__key: str = ""
+        try:
+            self.__key: str = self.__key
+        except:
+            self.__key = ""
         self.__exec_name: str
         self.__has_action_or_listener_executed: bool = False
 
+    def init(self):
+        """
+        Normalised __init__ without params definition.
+
+        Executed after __init__, usefull for adding logic that should be
+        executed on init but without need to rewrite/list init params.
+        """
+        pass
+
     @property
     def key(self) -> str:
         return self.__key
@@ -676,7 +823,7 @@ class Component(metaclass=ComponentMeta):
         """
         return dict()
 
-    def inject_into(self, component: "Component") -> Dict[str, Any]:
+    def inject_into(self, cconfig: "ComponentConfig") -> Dict[str, Any]:
         """
         inject_into params are used to inject  params into child component.
         This params usually defines values required by child compoennt that are
@@ -702,12 +849,15 @@ class Component(metaclass=ComponentMeta):
         if some param is injected it value will not be send to client nor acepted from x-jembe
         http request and JembeError will be raised.
         """
+        #  Not supportting this becouse it will force reinitialise of all childrens if
+        #   parent component action changes state regardles if parent injects params with
+        #   this method
         return dict()
 
     def isinjected(self, param_name: str) -> bool:
         return param_name in self._jembe_injected_params_names
 
-    def display(self) -> Union[str, "Response"]:
+    def display(self) -> "DisplayResponse":
         return self.render_template()
 
     def render_template(
@@ -775,15 +925,17 @@ class Component(metaclass=ComponentMeta):
                 )
             },
             # command to render subcomponents
-            "component": self.component,
-            "component_merge": self.component_merge,
-            "component_reset": self.component_reset,
+            "component": self._jinja2_component,
+            "component_reset": self._jinja2_component_reset,
             # add helpers
             "_config": self._config,
         }
 
-    def _component_template_tag(
-        self, merge_existing_params: bool, name: Optional[str] = None, **kwargs
+    def _component_reference(
+        self,
+        kwargs: Dict[str, Any],
+        name: Optional[str] = None,
+        merge_existing_params: bool = True,
     ) -> "ComponentReference":
         if name is None:
             try:
@@ -792,24 +944,31 @@ class Component(metaclass=ComponentMeta):
                 raise JembeError("Previous component renderer is not set")
         else:
             self.__prev_sub_component_renderer: "ComponentReference" = (
-                ComponentReference(self.exec_name, name, kwargs, merge_existing_params)
+                ComponentReference.factory(
+                    self.exec_name, name, kwargs, merge_existing_params
+                )
             )
             return self.__prev_sub_component_renderer
 
-    def component(self, name: Optional[str] = None, **kwargs) -> "ComponentReference":
-        return self._component_template_tag(
-            self._jembe_merged_existing_params, name, **kwargs
-        )
-
-    def component_merge(
-        self, name: Optional[str] = None, **kwargs
+    def component(
+        self, _jmb_component_name: str = ".", **kwargs
     ) -> "ComponentReference":
-        return self._component_template_tag(True, name, **kwargs)
+        return self._component_reference(kwargs, _jmb_component_name)
 
     def component_reset(
-        self, name: Optional[str] = None, **kwargs
+        self, _jmb_component_name: str = ".", **kwargs
     ) -> "ComponentReference":
-        return self._component_template_tag(False, name, **kwargs)
+        return self._component_reference(kwargs, _jmb_component_name, False)
+
+    def _jinja2_component(
+        self, _jmb_component_name: Optional[str] = None, **kwargs
+    ) -> "ComponentReference":
+        return self._component_reference(kwargs, _jmb_component_name)
+
+    def _jinja2_component_reset(
+        self, _jmb_component_name: Optional[str] = None, **kwargs
+    ) -> "ComponentReference":
+        return self._component_reference(kwargs, _jmb_component_name, False)
 
     def emit(self, name: str, **params) -> "EmitCommand":
         processor = get_processor()
@@ -820,3 +979,64 @@ class Component(metaclass=ComponentMeta):
     def get_storage(self, storage_name: Optional[str] = None) -> "Storage":
         processor = get_processor()
         return processor.jembe.get_storage(storage_name)
+
+    def redirect_to(self, component_ref: "ComponentReference"):
+        """
+        Redirects request to another component removing all unecessory
+        components from processing.
+        """
+        processor = get_processor()
+        # Remove all unecessary components that should not be needed after
+        # redirect by finding first component that is parent to both this compoment
+        # and redirected to compomennt (have same exec_name and same state)
+        # and remove all existing components and its assciated commands from
+        # jembe processors
+        root_cr = component_ref.root_renderer
+        processor.remove_component(
+            root_cr.exec_name,
+            self.exec_name.startswith("{}/".format(root_cr.exec_name)),
+        )
+
+        # Creates new commands to init and display for redirect components
+        # and puts in processenig que
+        component_ref()
+
+    def ac_allow(self, *action_names):
+        """
+        Allow execution of listed actions.
+        If no acction is listed allow execution of all actions.
+        """
+        if len(action_names) == 0:
+            self._jembe_disabled_actions = []
+        else:
+            for a in action_names:
+                try:
+                    self._jembe_disabled_actions.remove(a)
+                except ValueError:
+                    pass
+
+    def ac_deny(self, *action_names):
+        """
+        Deny execution of listed actions.
+        If no action is listed or "display" is listed
+        whole component is not accesible (__init__ will raise NotFound)
+        """
+        if len(action_names) == 0:
+            self._jembe_disabled_actions = [ComponentConfig.DEFAULT_DISPLAY_ACTION]
+            for a in self._config.component_actions.keys():
+                self._jembe_disabled_actions.append(a)
+        else:
+            for a in action_names:
+                if a not in self._jembe_disabled_actions and (
+                    a in self._config.component_actions.keys()
+                    or a == ComponentConfig.DEFAULT_DISPLAY_ACTION
+                ):
+                    self._jembe_disabled_actions.append(a)
+
+    def ac_check(self, action_name: Optional[str] = None) -> bool:
+        """
+        Apply access control rules
+        """
+        if ComponentConfig.DEFAULT_DISPLAY_ACTION in self._jembe_disabled_actions:
+            return False
+        return action_name not in self._jembe_disabled_actions
