@@ -24,7 +24,8 @@ from flask.globals import current_app
 from jinja2 import Undefined
 from lxml import etree
 from lxml.html import Element
-from flask import json, jsonify, Response, g
+from flask import json, jsonify, g
+from werkzeug import Response
 from .common import (
     convert_to_annotated_type,
     exec_name_to_full_name,
@@ -39,7 +40,7 @@ from .component_config import ComponentConfig, RedisplayFlag as RedisplayFlag
 if TYPE_CHECKING:  # pragma: no cover
     from .app import Jembe
     from flask import Request
-    from .component import Component, ComponentState
+    from .component import Component
     from .component_config import ComponentListener
 
 
@@ -159,8 +160,15 @@ class CallActionCommand(Command):
                     cconfig.full_name, self.action_name
                 )
             )
+        # check if action passes access control
+        if not component.ac_check(self.action_name):
+            raise ComponentConfig.DEFAULT_AC_EXCEPTION()
+
         # if component injects params into children save component state json hesh
-        if cconfig._inject_into_components is not None:
+        if (
+            cconfig._inject_into_components is not None
+            or cconfig.component_class._jembe_inject_into_overriden
+        ):
             self._component_state_before_execute = component.state.tojsondict(
                 component, True
             )
@@ -173,14 +181,15 @@ class CallActionCommand(Command):
         if action_result is None or (
             isinstance(action_result, bool) and action_result == True
         ):
-            # after executing action that returns True or None
-            # component should be rendered by executing display
-            self.processor.add_command(
-                CallDisplayCommand(
-                    self.component_exec_name, force=action_result == True
-                ),
-                end=True,
-            )
+            if self.component_exec_name in self.processor.components.keys():
+                # after executing action that returns True or None
+                # component should be rendered by executing display
+                self.processor.add_command(
+                    CallDisplayCommand(
+                        self.component_exec_name, force=action_result == True
+                    ),
+                    end=True,
+                )
         elif isinstance(action_result, bool) and action_result == False:
             # Do nothing
             pass
@@ -192,7 +201,9 @@ class CallActionCommand(Command):
         else:
             raise JembeError(
                 "Invalid action result type: {}.{} {}".format(
-                    cconfig.full_name, self.action_name, action_result,
+                    cconfig.full_name,
+                    self.action_name,
+                    action_result,
                 )
             )
 
@@ -261,6 +272,11 @@ class CallDisplayCommand(CallActionCommand):
     def execute(self) -> Optional["Response"]:
         if not self._redisplay_needed:
             return None
+
+        # check if action passes access control
+        if not self.force and not self._component.ac_check():
+            raise ComponentConfig.DEFAULT_AC_EXCEPTION()
+
         # execute action
         action_result = getattr(self._component, self.action_name)(
             *self.args, **self.kwargs
@@ -272,6 +288,7 @@ class CallDisplayCommand(CallActionCommand):
             self.processor.renderers[self.component_exec_name] = ComponentRender(
                 True,
                 self._component.state.tojsondict(self._component, True),
+                self._component._jembe_disabled_actions.copy(),
                 self._component.state._injected_params_names,
                 self._component.url,
                 self._component._config.changes_url,
@@ -280,14 +297,15 @@ class CallDisplayCommand(CallActionCommand):
         elif isinstance(action_result, Response):
             # TODO If self.component is component directly requested via http request
             # othervise raise JembeError
-            if not self.processor.renderers:
-                return action_result
-            else:
-                raise JembeError(
-                    "{} action should be first one to return responses if that response is not html string but full response object".format(
-                        self._component._config.full_name
-                    )
-                )
+            # if not self.processor.renderers:
+            #     return action_result
+            # else:
+            #     raise JembeError(
+            #         "{} action should be first one to return responses if that response is not html string but full response object".format(
+            #             self._component._config.full_name
+            #         )
+            #     )
+            return action_result
         elif not isinstance(action_result, str):
             # Display should return html string if not raise JembeError
             raise JembeError(
@@ -300,7 +318,9 @@ class CallDisplayCommand(CallActionCommand):
         else:
             raise JembeError(
                 "Invalid display result type: {}.{} {}".format(
-                    self._component._config.full_name, self.action_name, action_result,
+                    self._component._config.full_name,
+                    self.action_name,
+                    action_result,
                 )
             )
         return None
@@ -328,7 +348,10 @@ class CallDisplayCommand(CallActionCommand):
 
 class CallListenerCommand(Command):
     def __init__(
-        self, component_exec_name: str, listener_name: str, event: "Event",
+        self,
+        component_exec_name: str,
+        listener_name: str,
+        event: "Event",
     ):
         super().__init__(component_exec_name)
         self.listener_name = listener_name
@@ -362,7 +385,9 @@ class CallListenerCommand(Command):
         else:
             raise JembeError(
                 "Invalid listener result type: {}.{} {}".format(
-                    cconfig.full_name, self.listener_name, listener_result,
+                    cconfig.full_name,
+                    self.listener_name,
+                    listener_result,
                 )
             )
 
@@ -398,14 +423,14 @@ class EmitCommand(Command):
 
         if emit_event_to is:
 
-            - None -> match to every initialised component 
+            - None -> match to every initialised component
             - /compoent1.key1/compoent2.key2    -> compoenent with complete exec_name
             - ./component                       -> emit to direct child named "component" without key
             - ./component.*                     -> emit to direct child named "component" with any key
             - ./component.key                   -> emit to direct child named "component with key equals "key"
             - ./**/component[.[*|<key>]]        -> emit to child at any level
             - ..                                -> emit to parent
-            - ../component[.[*|<key>]]          -> emit to sibling 
+            - ../component[.[*|<key>]]          -> emit to sibling
             - /**/.                             -> emit to parent at any level
             - /**/component[.[*|<key>]]/**/.    -> emit to parent at any level named
             - //                                -> emit to my root page TODO
@@ -422,7 +447,7 @@ class EmitCommand(Command):
     def execute(self):
         """
         finds all components mathcing self.to that have registred listeners
-        whose source is matched to self.component.exec_name and calls matching 
+        whose source is matched to self.component.exec_name and calls matching
         listeners
         """
         self.event = Event(
@@ -469,18 +494,18 @@ class EmitCommand(Command):
 
         if pattern is:
 
-            - None -> match to every initialised component 
+            - None -> match to every initialised component
             - /compoent1.key1/compoent2.key2    -> compoenent with complete exec_name
-            - ./*                               -> direct children with or without key
-            - ./*.*                             -> direct children with any key
-            - ./**/*                            -> all children with or without key
-            - ./**/*.*                          -> all children with any key
-            - ./component                       -> match to direct child named "component" without key
-            - ./component.*                     -> match to direct child named "component" with any key
-            - ./component.key                   -> match to direct child named "component with key equals "key"
-            - ./**/component[.[*|<key>]]        -> match to child at any level
+            - *                               -> direct children with or without key
+            - *.*                             -> direct children with any key
+            - **/*                            -> all children with or without key
+            - **/*.*                          -> all children with any key
+            - component                       -> match to direct child named "component" without key
+            - component.*                     -> match to direct child named "component" with any key
+            - component.key                   -> match to direct child named "component with key equals "key"
+            - **/component[.[*|<key>]]        -> match to child at any level
             - ..                                -> match to parent
-            - ../component[.[*|<key>]]          -> match to sibling 
+            - ../component[.[*|<key>]]          -> match to sibling
             - /**/.                             -> match to parent at any level
             - /**/component[.[*|<key>]]/**/.    -> match to parent at any level named
             - //                                -> process event from root page
@@ -523,7 +548,11 @@ class EmitCommand(Command):
             # regular path match
             if pattern.startswith("/"):
                 pass
-            elif pattern.startswith("./") or pattern == ".":
+            elif pattern == ".":
+                pattern = pattern_exec_name
+            elif not (pattern.startswith(".") or pattern.startswith("/")):
+                pattern = "{}/{}".format(pattern_exec_name, pattern)
+            elif pattern.startswith("./"):
                 pattern = "{}{}".format(pattern_exec_name, pattern.lstrip("."))
             elif pattern.startswith(".."):
                 pattern_begins = pattern_exec_name.lstrip("/").split("/")
@@ -540,7 +569,7 @@ class EmitCommand(Command):
                 re.escape(pattern)
                 .replace(
                     "/\\*\\*/",
-                    "((/.*/)|(/))",  # ** replace wit regex to match compoente path
+                    "((/.*/)|(/))",  # ** replace wit regex to match component path
                 )
                 .replace("\\*\\.\\*", "[^./]+\\.[^./]+")
                 .replace("\\.\\*", "\\.[^./]+")  # replace with regex to match any key
@@ -571,7 +600,9 @@ class EmitCommand(Command):
             ):
                 commands.append(
                     CallListenerCommand(
-                        reemit_component.exec_name, listener_method_name, self.event,
+                        reemit_component.exec_name,
+                        listener_method_name,
+                        self.event,
                     )
                 )
         return commands
@@ -612,12 +643,16 @@ class EmitCommand(Command):
             )
 
         sources_to: tuple = tuple((None,)) if not source_to else source_to
-        listener_events: tuple = tuple(
-            (None,)
-        ) if not destination_listener.event_names else destination_listener.event_names
-        listener_sources: tuple = tuple(
-            (None,)
-        ) if not destination_listener.sources else destination_listener.sources
+        listener_events: tuple = (
+            tuple((None,))
+            if not destination_listener.event_names
+            else destination_listener.event_names
+        )
+        listener_sources: tuple = (
+            tuple((None,))
+            if not destination_listener.sources
+            else destination_listener.sources
+        )
 
         for s_to in sources_to:
             for dl_event in listener_events:
@@ -653,33 +688,46 @@ class InitialiseCommand(Command):
     @cached_property
     def _inject_into_params(self) -> Dict[str, Any]:
         parent_cconfig = self._cconfig.parent
-        injected_params = dict()
-        if parent_cconfig and parent_cconfig._inject_into_components:
-            injected_params = parent_cconfig._inject_into_components(
-                self.processor.components[parent_exec_name(self.component_exec_name)],
-                self._cconfig,
+        if parent_cconfig:
+            parent_component = self.processor.components[
+                parent_exec_name(self.component_exec_name)
+            ]
+            injected_params = (
+                parent_component.inject_into(self._cconfig)
+                if parent_component._jembe_inject_into_overriden
+                else dict()
             )
+            if parent_cconfig._inject_into_components:
+                injected_params.update(
+                    parent_cconfig._inject_into_components(
+                        parent_component,
+                        self._cconfig,
+                    )
+                )
             # clean up injected params
-            injected_params = {
-                key: value
-                for key, value in injected_params.items()
-                if key in self._cconfig.component_class._jembe_init_param_names
-            }
-        return injected_params
+            if injected_params:
+                injected_params = {
+                    key: value
+                    for key, value in injected_params.items()
+                    if key in self._cconfig.component_class._jembe_init_param_names
+                }
+            return injected_params
+        else:
+            return dict()
 
     def _must_do_init(self, is_accessible_run: bool):
         if self.component_exec_name in self.processor.components:
+            component = self.processor.components[self.component_exec_name]
             new_params = (
                 {}
                 if self.merge_existing_params
                 else {**self._cconfig.component_class._jembe_state_param_default_values}
             )
             new_params.update(
-                {**self.init_params, **self._inject_into_params,}
+                {**self.init_params, **self._inject_into_params, **component.inject()}
             )
             # if state params are same continue
             # else raise jembeerror until find better solution
-            component = self.processor.components[self.component_exec_name]
             has_new_params = False
             for k, v in new_params.items():
                 if k in component.state and v != component.state[k]:
@@ -725,16 +773,21 @@ class InitialiseCommand(Command):
                 else {**self._cconfig.component_class._jembe_state_param_default_values}
             )
             init_params.update(
-                {**existing_params, **self.init_params, **self._inject_into_params,}
+                {**existing_params, **self.init_params, **self._inject_into_params}
             )
+
             component = self._cconfig.component_class._jembe_init_(
                 self._cconfig,
+                self.component_exec_name,
                 list(self._inject_into_params.keys()),
                 self.merge_existing_params,
                 **init_params
             )
 
-            component.exec_name = self.component_exec_name
+            # check if action passes access control
+            if not component.ac_check():
+                raise ComponentConfig.DEFAULT_AC_EXCEPTION()
+
             self.initialised_component = component
             if not is_accessible_run:
                 self.processor.components[component.exec_name] = component
@@ -743,6 +796,7 @@ class InitialiseCommand(Command):
                     self.processor.renderers[component.exec_name] = ComponentRender(
                         False,
                         component.state.tojsondict(component, True),
+                        component._jembe_disabled_actions.copy(),
                         component.state._injected_params_names,
                         component.url,
                         component._config.changes_url,
@@ -777,6 +831,7 @@ class ComponentRender(NamedTuple):
 
     fresh: bool
     state_jsondict: Dict[str, Any]
+    disabled_actions: List[str]
     injected_params: List[str]
     url: Optional[str]
     changes_url: bool
@@ -794,7 +849,7 @@ class CommandsQue:
         """
         Adds new command into staginig commands que.
         if end is True command is added at the end of que (last to execute)
-        
+
         Staging que exist in order to allow commands generated by while
         execution one command to be internally ordered by appending at
         the beginig or the end of staging que
@@ -866,8 +921,8 @@ class CommandsQue:
 class Processor:
     """
     1. Will use deapest component.url from all components on the page as window.location
-    2. When any component action or listener returns False default action 
-       (display) will not be called 
+    2. When any component action or listener returns False default action
+       (display) will not be called
     """
 
     def __init__(self, jembe: "Jembe", component_full_name: str, request: "Request"):
@@ -915,9 +970,9 @@ class Processor:
         self, command_data: dict, is_component: bool = False
     ) -> "Command":
         """Process data received by json and build commands
-        
-            is_component - mark that command_data is received in components section of x-jembe request
-                        describing current compoents and their state displayed to the user
+
+        is_component - mark that command_data is received in components section of x-jembe request
+                    describing current compoents and their state displayed to the user
         """
 
         if is_component:
@@ -983,21 +1038,23 @@ class Processor:
             )
 
             self.add_command(
-                CallDisplayCommand(exec_names[-1]), end=True,
+                CallDisplayCommand(exec_names[-1]),
+                end=True,
             )
             for exec_name in exec_names[:-1]:
                 self.add_command(
-                    CallDisplayCommand(exec_name), end=True,
+                    CallDisplayCommand(exec_name),
+                    end=True,
                 )
 
     def __create_commands_from_url_path(
         self, component_full_name: str, to_be_initialised: List[str]
     ) -> List[str]:
-        """ 
-            inits components from request url_path 
-            if component with same exec_name is not already initialised
+        """
+        inits components from request url_path
+        if component with same exec_name is not already initialised
 
-            returns exec names from root to component_full_name
+        returns exec names from root to component_full_name
         """
         from .component import Component
 
@@ -1147,7 +1204,7 @@ class Processor:
             return None
 
     def execute_initialise_command_successfully(
-        self, command: "InitialiseCommand"
+        self, command: "InitialiseCommand", *additional_components: "Component"
     ) -> Tuple[bool, Optional["Component"]]:
         """
         Directly out of commands que execute initialise command
@@ -1171,6 +1228,11 @@ class Processor:
         # execute initialise command without running before or after commands
         backup_current_staging_commands = self._staging_commands
         self._staging_commands = CommandsQue(self.jembe)
+        backup_current_components = self.components.copy()
+        # add additional components into components
+        for acomp in additional_components:
+            self.components[acomp.exec_name] = acomp
+
         cmd: "InitialiseCommand" = (
             command if command.is_mounted else command.mount(self)
         )
@@ -1179,6 +1241,7 @@ class Processor:
         except JembeError as jmb_error:
             # JembeError are exceptions raised by jembe
             # and thay indicate bad usage of framework
+            self.components = backup_current_components
             raise jmb_error
         except Exception as exc:
             self._raised_exception_on_initialise[cmd.component_exec_name] = deepcopy(
@@ -1188,14 +1251,18 @@ class Processor:
 
             # restore _staging_commands
             self._staging_commands = backup_current_staging_commands
+            self.components = backup_current_components
             if current_app.debug or current_app.testing:
+                # import traceback
+                # traceback.print_exc()
                 current_app.logger.warning(
                     "Exception when initialising component out of proccessing que {}: {}".format(
-                        cmd, exc
+                        cmd, exc.__repr__()
                     )
                 )
             return (False, None)
         self._staging_commands = backup_current_staging_commands
+        self.components = backup_current_components
         return (True, cmd.initialised_component)
 
     def _handle_exception_in_command(self, command: "Command", exc: "Exception"):
@@ -1203,9 +1270,9 @@ class Processor:
         Exception that ocure while executing command (call action, display, listener invocation etc)
         are handled in following way.
 
-        Event('_exception', source=command.component_exec_name, params=dict(exception=exc, handled=False)) is created 
-        and emited to all its parent components from top to bottom. 
-        This is only time when ordering for receiving messages is guaranted, so that root components can know if 
+        Event('_exception', source=command.component_exec_name, params=dict(exception=exc, handled=False)) is created
+        and emited to all its parent components from top to bottom.
+        This is only time when ordering for receiving messages is guaranted, so that root components can know if
         exception is already handled by its children.
 
         Parent component can use regular listener to catch event do whatever thay want. @listener('_exception')
@@ -1222,9 +1289,15 @@ class Processor:
             EmitCommand(
                 command.component_exec_name,
                 SystemEvents.EXCEPTION.value,
-                dict(exception=exc, handled=False),
+                dict(
+                    exception=exc,
+                    handled=False,
+                    in_action=command.action_name
+                    if isinstance(command, CallActionCommand)
+                    else None,
+                ),
             )
-            .to("/**/.")
+            .to((".", "/**/."))
             .mount(self)
         )
         emit_command.execute()
@@ -1269,7 +1342,15 @@ class Processor:
             ajax_responses = []
             for (
                 exec_name,
-                (fresh, state_jsondict, injected_params_names, url, changes_url, html),
+                (
+                    fresh,
+                    state_jsondict,
+                    disabled_actions,
+                    injected_params_names,
+                    url,
+                    changes_url,
+                    html,
+                ),
             ) in self.renderers.items():
                 if fresh:
                     ajax_responses.append(
@@ -1283,13 +1364,13 @@ class Processor:
                             dom=html,
                             url=url,
                             changesUrl=changes_url,
-                            actions=[
-                                action_name
+                            actions={
+                                action_name: action_name not in disabled_actions
                                 for action_name in self.jembe.get_component_config(
                                     exec_name
                                 ).component_actions.keys()
                                 if action_name != ComponentConfig.DEFAULT_DISPLAY_ACTION
-                            ],
+                            },
                         )
                     )
             return jsonify(ajax_responses)
@@ -1306,10 +1387,12 @@ class Processor:
                     },
                     url,
                     changes_url,
+                    disabled_actions,
                 )
                 for exec_name, (
                     fresh,
                     state_jsondict,
+                    disabled_actions,
                     injected_params_names,
                     url,
                     changes_url,
@@ -1367,6 +1450,7 @@ class Processor:
         state_jsondict: Dict[str, Any],
         url: str,
         changes_url: bool,
+        disabled_actions: List[str],
     ):  # -> "lxml.html.HtmlElement":
         """
         Adds dom attrs to html.
@@ -1380,13 +1464,13 @@ class Processor:
                 "jmb-data",
                 json.dumps(
                     dict(
-                        actions=[
-                            action_name
+                        actions={
+                            action_name: action_name not in disabled_actions
                             for action_name in self.jembe.get_component_config(
                                 exec_name
                             ).component_actions.keys()
                             if action_name != ComponentConfig.DEFAULT_DISPLAY_ACTION
-                        ],
+                        },
                         changesUrl=changes_url,
                         state=state_jsondict,
                         url=url,
@@ -1454,3 +1538,26 @@ class Processor:
                 ts.rmdir("uploads")
             except:
                 pass
+
+    def remove_component(self, exec_name: str, only_children: bool = False):
+        def match_exec_name(match: str) -> bool:
+            if match.startswith("{}/".format(exec_name)):
+                return True
+            if only_children == False and match == exec_name:
+                return True
+            return False
+
+        self._commands = deque(
+            (
+                c
+                for c in self._commands
+                if not match_exec_name(c.component_exec_name)
+                or isinstance(c, EmitCommand)
+            )
+        )
+        self.renderers = {
+            en: r for en, r in self.renderers.items() if not match_exec_name(en)
+        }
+        self.components = {
+            en: c for en, c in self.components.items() if not match_exec_name(en)
+        }
